import os
from collections import Counter
from tqdm import tqdm

nc = 80
cats = {i:str(i) for i in range(nc)}
clses = []
lbl_rt = r'F:\TempFiles\ForPyCharm\pythonProject\coco128\labels\train2017'
lbls = os.listdir(lbl_rt)
for lbl in tqdm(lbls):
    lbl_f = fr'{lbl_rt}\{lbl}'
    with open(lbl_f, 'r', encoding='utf8') as f:
        lines = f.readlines()
    for line in lines:
        if not line:
            continue
        line = line.strip().split()
        try:
            clses.append(int(line[0]))
        except:
            continue

counter = Counter(clses)
print(counter)





import torch
import torch.nn as nn
import numpy as np


def modelsize(model, input, type_size=4):
    para = sum([np.prod(list(p.size())) for p in model.parameters()])
    # print('Model {} : Number of params: {}'.format(model._get_name(), para))
    print('Model {} : params: {:4f}M'.format(model._get_name(), para * type_size / 1000 / 1000))

    input_ = input.clone()
    input_.requires_grad_(requires_grad=False)

    mods = list(model.modules())
    out_sizes = []

    for i in range(1, len(mods)):
        m = mods[i]
        if isinstance(m, nn.ReLU):
            if m.inplace:
                continue
        out = m(input_)
        out_sizes.append(np.array(out.size()))
        input_ = out

    total_nums = 0
    for i in range(len(out_sizes)):
        s = out_sizes[i]
        nums = np.prod(np.array(s))
        total_nums += nums

    # print('Model {} : Number of intermedite variables without backward: {}'.format(model._get_name(), total_nums))
    # print('Model {} : Number of intermedite variables with backward: {}'.format(model._get_name(), total_nums*2))
    print('Model {} : intermedite variables: {:3f} M (without backward)'
          .format(model._get_name(), total_nums * type_size / 1000 / 1000))
    print('Model {} : intermedite variables: {:3f} M (with backward)'
          .format(model._get_name(), total_nums * type_size*2 / 1000 / 1000))




# -*- coding: utf-8 -*-
"""
@Time: 2024-01-23 20:30
@Auth: xjjxhxgg
@File: __init__.py
@IDE: PyCharm
@Motto: xhxgg
"""
from .format import (
    coco2yolo, yolo2coco, xml2yolo, yolo2coco_n,
    yolo_parser, yolo_obb_parser, coco_parser, xml_parser, xml_parser_obb,
    tif2jpg,
)
from .visualize import visualize

compenents = {
    'coco2yolo': coco2yolo,
    'yolo2coco': yolo2coco,
    'xml2yolo': xml2yolo,
    'yolo2coco_n': yolo2coco_n,
    'yolo_parser': yolo_parser,
    'yolo_obb_parser': yolo_obb_parser,
    'coco_parser': coco_parser,
    'xml_parser': xml_parser,
    'xml_parser_obb': xml_parser_obb,
    'tif2jpg': tif2jpg,
    'visualize': visualize
}



# -*- coding: utf-8 -*-
"""
@Time: 2024-02-03 17:55
@Auth: xjjxhxgg
@File: download.py
@IDE: PyCharm
@Motto: xhxgg
"""
import requests


def get(url, file, mode='wb'):
    """
    Given a URL, this function will download the content and save it to the specified location as the specified format.
    :param url: the URL of the content to be downloaded
    :param file: the location where the content will be saved, including the file name and extension name
    :param mode: the format of the content to be saved, 'wb' for binary file and 'w' for text file
    :return: None
    """
    response = requests.get(url)
    if mode == 'wb':
        with open(file, mode) as f:
            f.write(response.content)
    else:
        with open(file, mode, encoding='utf-8') as f:
            f.write(response.text)



import os
import numpy as np
import torch
from tqdm import tqdm
from pathlib import Path
from matplotlib import pyplot as plt
import warnings
from ultralytics.utils import LOGGER, SimpleClass, TryExcept, plt_settings
from ultralytics.utils.metrics import box_iou, SimpleClass, Metric, DetMetrics, smooth, compute_ap, ap_per_class
device = torch.device(0)

def load_yolo_labels(file_path):
    with open(file_path, 'r') as f:
        labels = [[float(i) for i in line.strip().split()] for line in f.readlines()]
    labels = [(label[0], label[1] - label[3] / 2, label[2] - label[4] / 2, label[1] + label[3] / 2, label[2] + label[4] / 2) for label in labels]
    labels = np.array([(int(label[0]), float(label[1]), float(label[2]), float(label[3]), float(label[4])) for label in labels])
    cls = labels[:, 0]
    bbox = labels[:, 1:]
    return labels, cls, bbox
def load_prd_with_conf(file_path):
    with open(file_path, 'r') as f:
        labels = [[float(i) for i in line.strip().split()] for line in f.readlines()]
    try:
        labels = [(label[0], label[1] - label[3] / 2, label[2] - label[4] / 2, label[1] + label[3] / 2, label[2] + label[4] / 2, label[5]) for label in labels]
    except:
        labels = [(label[0], label[1] - label[3] / 2, label[2] - label[4] / 2, label[1] + label[3] / 2, label[2] + label[4] / 2, 1) for label in labels]
    labels = np.array([(int(label[0]), float(label[1]), float(label[2]), float(label[3]), float(label[4]), float(label[5])) for label in labels])
    cls = labels[:, 0]
    bbox = labels[:, 1:5]
    conf = labels[:, 5]
    return labels, cls, bbox, conf


def calculate_tps(all_ground_truths, all_detections, thrhds=None):
    if thrhds is None:
        thrhds = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1]
    tps = []
    for thrhd in tqdm(thrhds):
        tps.append(calculate_tp(all_ground_truths, all_detections, thrhd))
    return np.array(tps).transpose(1, 0)

def calculate_tp(all_ground_truths, all_detections, thrhd):
    TP_for_each_file = []
    for ground_truths, detections in zip(all_ground_truths, all_detections):
        matched = set()
        TP = []
        for det in detections:
            found_match = False
            for idx, gt in enumerate(ground_truths):
                if idx in matched:
                    continue
                if det[0] == gt[0]: # class matched
                    if box_iou(torch.tensor(gt[None, 1:], device=device), torch.tensor(det[None, 1:], device=device)) >= thrhd:
                        TP.append(True)
                        TP_for_each_file.append(True)
                        matched.add(idx)
                        found_match = True
                        break
            if not found_match:
                TP.append(False)
                TP_for_each_file.append(False)

    return np.array(TP_for_each_file)

if __name__ == '__main__':
    # names = {0:'ship', 1:'plane', 2:'vehicle'}
    names = {i:str(i) for i in range(80)}
    lbl_rt = r'F:\611\coco128\labels\train2017'
    prd_rt = r'F:\611\coco128\preds'

    lbl_files = [os.path.join(lbl_rt, lbl) for lbl in os.listdir(lbl_rt)]
    prd_files = [os.path.join(prd_rt, lbl) for lbl in os.listdir(prd_rt)]

    lbl_files = [fr'{lbl_rt}\000000000357.txt', fr'{lbl_rt}\000000000472.txt']
    # prd_files = [fr'{lbl_rt}\000000000357.txt', fr'{lbl_rt}\000000000472.txt']
    prd_files = [fr'{prd_rt}\000000000357.txt', fr'{prd_rt}\000000000472.txt']
    metrics = Metric()

    detections_for_each_file = []
    gt_cls_for_each_file = np.array([])
    gt_box_for_each_file = []

    for lbl_file in tqdm(lbl_files):
        lbls, gt_cls, gt_box = load_yolo_labels(lbl_file)
        detections_for_each_file.append(lbls)
        gt_cls_for_each_file = np.append(gt_cls_for_each_file, gt_cls)
        gt_box_for_each_file.append(gt_box)

    predictions_for_each_file = []
    pd_cls_for_each_file = np.array([])
    pd_box_for_each_file = np.array([])
    pd_conf_for_each_file = np.array([])

    for prd_file in tqdm(prd_files):
        prds, pd_cls, pd_box, conf = load_prd_with_conf(prd_file)
        predictions_for_each_file.append(prds[:, :-1])
        pd_cls_for_each_file = np.append(pd_cls_for_each_file, pd_cls)
        pd_box_for_each_file = np.append(pd_box_for_each_file, pd_box)
        pd_conf_for_each_file = np.append(pd_conf_for_each_file, conf)
    pd_conf_for_each_file = np.array(pd_conf_for_each_file).reshape(-1)
    gt_cls_for_each_file = np.array(gt_cls_for_each_file).reshape(-1)
    pd_cls_for_each_file = np.array(pd_cls_for_each_file).reshape(-1)

    TP_for_each_file = calculate_tps(detections_for_each_file, predictions_for_each_file)
    res = ap_per_class(TP_for_each_file, pd_conf_for_each_file, pd_cls_for_each_file,gt_cls_for_each_file, names=names)
    tp, fp, p, r, f1, ap, unique_classes, p_curve, r_curve, f1_curve, x, prec_values = res

    metrics.update(res[2:])
    print(res)





import gc
import datetime
import inspect

import torch
import numpy as np

dtype_memory_size_dict = {
    torch.float64: 64/8,
    torch.double: 64/8,
    torch.float32: 32/8,
    torch.float: 32/8,
    torch.float16: 16/8,
    torch.half: 16/8,
    torch.int64: 64/8,
    torch.long: 64/8,
    torch.int32: 32/8,
    torch.int: 32/8,
    torch.int16: 16/8,
    torch.short: 16/6,
    torch.uint8: 8/8,
    torch.int8: 8/8,
}
# compatibility of torch1.0
if getattr(torch, "bfloat16", None) is not None:
    dtype_memory_size_dict[torch.bfloat16] = 16/8
if getattr(torch, "bool", None) is not None:
    dtype_memory_size_dict[torch.bool] = 8/8 # pytorch use 1 byte for a bool, see https://github.com/pytorch/pytorch/issues/41571

def get_mem_space(x):
    try:
        ret = dtype_memory_size_dict[x]
    except KeyError:
        print(f"dtype {x} is not supported!")
    return ret

class MemTracker(object):
    """
    Class used to track pytorch memory usage
    Arguments:
        detail(bool, default True): whether the function shows the detail gpu memory usage
        path(str): where to save log file
        verbose(bool, default False): whether show the trivial exception
        device(int): GPU number, default is 0
    """
    def __init__(self, detail=True, path='', verbose=False, device=0):
        self.print_detail = detail
        self.last_tensor_sizes = set()
        self.gpu_profile_fn = path + f'{datetime.datetime.now():%d-%b-%y-%H:%M:%S}-gpu_mem_track.txt'
        self.verbose = verbose
        self.begin = True
        self.device = device

    def get_tensors(self):
        for obj in gc.get_objects():
            try:
                if torch.is_tensor(obj) or (hasattr(obj, 'data') and torch.is_tensor(obj.data)):
                    tensor = obj
                else:
                    continue
                if tensor.is_cuda:
                    yield tensor
            except Exception as e:
                if self.verbose:
                    print('A trivial exception occured: {}'.format(e))

    def get_tensor_usage(self):
        sizes = [np.prod(np.array(tensor.size())) * get_mem_space(tensor.dtype) for tensor in self.get_tensors()]
        return np.sum(sizes) / 1024**2

    def get_allocate_usage(self):
        return torch.cuda.memory_allocated() / 1024**2

    def clear_cache(self):
        gc.collect()
        torch.cuda.empty_cache()

    def print_all_gpu_tensor(self, file=None):
        for x in self.get_tensors():
            print(x.size(), x.dtype, np.prod(np.array(x.size()))*get_mem_space(x.dtype)/1024**2, file=file)

    def track(self):
        """
        Track the GPU memory usage
        """
        frameinfo = inspect.stack()[1]
        where_str = frameinfo.filename + ' line ' + str(frameinfo.lineno) + ': ' + frameinfo.function

        with open(self.gpu_profile_fn, 'a+') as f:

            if self.begin:
                f.write(f"GPU Memory Track | {datetime.datetime.now():%d-%b-%y-%H:%M:%S} |"
                        f" Total Tensor Used Memory:{self.get_tensor_usage():<7.1f}Mb"
                        f" Total Allocated Memory:{self.get_allocate_usage():<7.1f}Mb\n\n")
                self.begin = False

            if self.print_detail is True:
                ts_list = [(tensor.size(), tensor.dtype) for tensor in self.get_tensors()]
                new_tensor_sizes = {(type(x),
                                    tuple(x.size()),
                                    ts_list.count((x.size(), x.dtype)),
                                    np.prod(np.array(x.size()))*get_mem_space(x.dtype)/1024**2,
                                    x.dtype) for x in self.get_tensors()}
                for t, s, n, m, data_type in new_tensor_sizes - self.last_tensor_sizes:
                    f.write(f'+ | {str(n)} * Size:{str(s):<20} | Memory: {str(m*n)[:6]} M | {str(t):<20} | {data_type}\n')
                for t, s, n, m, data_type in self.last_tensor_sizes - new_tensor_sizes:
                    f.write(f'- | {str(n)} * Size:{str(s):<20} | Memory: {str(m*n)[:6]} M | {str(t):<20} | {data_type}\n')

                self.last_tensor_sizes = new_tensor_sizes

            f.write(f"\nAt {where_str:<50}"
                    f" Total Tensor Used Memory:{self.get_tensor_usage():<7.1f}Mb"
                    f" Total Allocated Memory:{self.get_allocate_usage():<7.1f}Mb\n\n")



# -*- coding: utf-8 -*-
"""
@Time: 2024-01-23 20:20
@Auth: xjjxhxgg
@File: coco_to_yolo.py
@IDE: PyCharm
@Motto: xhxgg
"""

import json
import math
import os

import cv2


def coco2yolo(json_path, yolo_root, cat_reid=False):
    ann_file = json.load(open(json_path, 'r'))
    images = ann_file['images']
    annotations = ann_file['annotations']
    categories = ann_file['categories']
    categories_reid = [cat['id'] for cat in categories]
    categories_reid.sort()
    img_ann_map = {}
    for img in images:
        img_ann_map[img['id']] = []
    for ann in annotations:
        img_ann_map[ann['image_id']].append(ann)
    file_name_key = 'file_name' if 'file_name' in images[0] else 'filename'
    for img in images:
        img_name = img[file_name_key]
        img_width = img['width']
        img_height = img['height']
        img_id = img['id']
        img_anns = img_ann_map[img_id]
        yolo_txt_path = os.path.join(yolo_root, img_name.split('.')[0] + '.txt')
        with open(yolo_txt_path, 'w') as f:
            for ann in img_anns:
                cat_id = ann['category_id']
                if cat_reid:
                    cat_id = categories_reid.index(cat_id)
                bbox = ann['bbox']
                x = bbox[0]
                y = bbox[1]
                w = bbox[2]
                h = bbox[3]
                x_center = x + w / 2
                y_center = y + h / 2
                x_center /= img_width
                y_center /= img_height
                w /= img_width
                h /= img_height
                f.write(str(cat_id) + ' ' + str(x_center) + ' ' + str(y_center) + ' ' + str(w) + ' ' + str(h) + '\n')


def yolo2coco(yolo_root, json_path, categories, img_root, img_id=0, ann_id=0, info='', licenses='', img_suffix='.jpg'):
    ann_file = open(json_path, 'w')
    images = []
    annotations = []
    for root, dirs, files in os.walk(yolo_root):
        for file in files:
            if file.endswith('.txt'):
                img_id += 1
                img_name = file.split('.')[0] + img_suffix
                img_file = fr'{img_root}\{img_name}'
                img = cv2.imread(img_file)
                height, width, _ = img.shape
                img = {'file_name': img_name, 'height': height, 'width': width, 'id': img_id}
                images.append(img)
                txt_path = os.path.join(root, file)
                with open(txt_path, 'r') as f:
                    for line in f.readlines():
                        line = line.strip()
                        if line == '':
                            continue
                        cat, x, y, w, h = line.split(' ')
                        x = float(x)
                        y = float(y)
                        w = float(w)
                        h = float(h)
                        x *= width
                        y *= height
                        w *= width
                        h *= height
                        xmin = int(x - w / 2)
                        ymin = int(y - h / 2)
                        cat = int(cat)
                        bbox = [xmin, ymin, w, h]
                        ann_id += 1
                        ann = {'id': ann_id, 'image_id': img_id, 'category_id': cat, 'bbox': bbox, 'area': w * h, 'iscrowd': 0}
                        annotations.append(ann)
    ann_file.write(json.dumps(
        {'images': images, 'annotations': annotations, 'categories': categories, 'info': info, 'licenses': licenses}))
    ann_file.close()

def yolo2coco_n(yolo_root, json_path, categories, n, img_id=0, ann_id=0, info='', licenses=''):
    """

    :param yolo_root:
    :param json_path:
    :param categories:
    :param n: train, val or test to specify which sub-dataset to transfer
    :param img_id:
    :param ann_id:
    :param info:
    :param licenses:
    :return:
    """
    # img_root = os.path.join(yolo_root, 'images', n)
    img_root = fr'{yolo_root}\images\{n}'
    # lbl_root = os.path.join(yolo_root, 'labels', n)
    lbl_root = fr'{yolo_root}\labels\{n}'
    files = os.listdir(lbl_root)
    ann_file = open(json_path, 'w')
    images = []
    annotations = []
    for file in files:
        if file.endswith('.txt'):
            img_id += 1
            img_name = file.split('.')[0] + '.jpg'
            img_file = os.path.join(img_root, img_name)
            img = cv2.imread(img_file)
            height, width, _ = img.shape
            img = {'file_name': img_name, 'height': height, 'width': width, 'id': img_id}
            images.append(img)
            txt_path = os.path.join(lbl_root, file)
            with open(txt_path, 'r') as f:
                for line in f.readlines():
                    line = line.strip()
                    if line == '':
                        continue
                    cat, x, y, w, h = line.split(' ')
                    x = float(x)
                    y = float(y)
                    w = float(w)
                    h = float(h)
                    x *= width
                    y *= height
                    w *= width
                    h *= height
                    xmin = int(x - w / 2)
                    ymin = int(y - h / 2)
                    cat = int(cat)
                    bbox = [xmin, ymin, w, h]
                    ann_id += 1
                    ann = {'id': ann_id, 'image_id': img_id, 'category_id': cat, 'bbox': bbox, 'area': h * w, 'iscrowd': 0}
                    annotations.append(ann)
    ann_file.write(json.dumps(
        {'images': images, 'annotations': annotations, 'categories': categories, 'info': info, 'licenses': licenses}))
    ann_file.close()

def coco_parser(ann_path):
    '''
    return {'images': images, 'categories': categories, 'img_ann_map': img_ann_map}
    :param ann_path:
    :return:
    '''
    ann = json.load(open(ann_path, 'r'))
    images = ann['images']
    annotations = ann['annotations']
    categories = ann['categories']
    img_ann_map = {}
    for img in images:
        img_id = img['id']
        img_ann_map[img_id] = []
        for ann in annotations:
            if ann['image_id'] == img_id:
                img_ann_map[img_id].append(ann)

    return {'images': images, 'categories': categories, 'img_ann_map': img_ann_map}


def yolo_parser(yolo_path, img_w=None, img_h=None):
    """
    return bboxes [{'name':name,'bbox':[xmin,ymin,xmax,ymax]},...] or [{'name':name,'bbox':[cx,cy,w,h]},...]
    :param yolo_path:
    :param img_w: image width
    :param img_h: image height
    if img_w and img_h are not None, the bbox will be scaled to the original image size,
    otherwise the bbox is the relative size
    :return:
    """
    bboxes = []
    with open(yolo_path, 'r') as f:
        for line in f.readlines():
            line = line.strip()
            if line == '':
                continue
            cat, x, y, w, h = line.split(' ')
            cat = int(cat)
            cx = float(x)
            cy = float(y)
            w = float(w)
            h = float(h)
            if img_w and img_h:
                xmin = int((cx - w / 2) * img_w)
                ymin = int((cy - h / 2) * img_h)
                xmax = int((cx + w / 2) * img_w)
                ymax = int((cy + h / 2) * img_h)
                bboxes.append({'name': cat, 'bbox': [xmin, ymin, xmax, ymax]})
            else:
                bboxes.append({'name': cat, 'bbox': [cx, cy, w, h]})
    return bboxes

def yolo_obb_parser(yolo_path, img_w=None, img_h=None):
    """
    return bboxes [{'name':name,'bbox':[xmin,ymin,xmax,ymax]},...] or [{'name':name,'bbox':[cx,cy,w,h]},...]
    :param yolo_path:
    :param img_w: image width
    :param img_h: image height
    if img_w and img_h are not None, the bbox will be scaled to the original image size,
    otherwise the bbox is the relative size
    :return:
    """
    bboxes = []
    with open(yolo_path, 'r') as f:
        for line in f.readlines():
            line = line.strip()
            if line == '':
                continue
            cat, xlt, ylt, xrt, yrt, xrb, yrb, xlb, ylb = line.split(' ')
            cat = int(cat)
            xlt = float(xlt)
            ylt = float(ylt)
            xrt = float(xrt)
            yrt = float(yrt)
            xrb = float(xrb)
            yrb = float(yrb)
            xlb = float(xlb)
            ylb = float(ylb)
            obbw = math.sqrt((xlt - xrt) ** 2 + (ylt - yrt) ** 2)
            obbh = math.sqrt((xlt - xlb) ** 2 + (ylt - ylb) ** 2)
            # calculate the minimum bounding rectangle
            brw = max(xlt, xrt, xrb, xlb) - min(xlt, xrt, xrb, xlb)
            brh = max(ylt, yrt, yrb, ylb) - min(ylt, yrt, yrb, ylb)
            brcx = (xlt + xrt + xrb + xlb) / 4
            brcy = (ylt + yrt + yrb + ylb) / 4
            if img_w and img_h:
                brxmin = int((brcx - brw / 2) * img_w)
                brymin = int((brcy - brh / 2) * img_h)
                brxmax = int((brcx + brw / 2) * img_w)
                brymax = int((brcy + brh / 2) * img_h)
                xlt = int(xlt * img_w)
                ylt = int(ylt * img_h)
                xrt = int(xrt * img_w)
                yrt = int(yrt * img_h)
                xrb = int(xrb * img_w)
                yrb = int(yrb * img_h)
                xlb = int(xlb * img_w)
                ylb = int(ylb * img_h)
                obbw = int(obbw * img_w)
                obbh = int(obbh * img_h)
                bboxes.append({'name': cat, 'br': [brxmin, brymin, brxmax, brymax], 'bbox': [xlt, ylt, xrt, yrt, xrb, yrb, xlb, ylb, obbw, obbh]})
            else:
                bboxes.append({'name': cat, 'br': [brcx, brcy, brw, brh], 'bbox': [xlt, ylt, xrt, yrt, xrb, yrb, xlb, ylb, obbw, obbh]})

    return bboxes


def xml_parser(ann_path):
    '''
    return bboxes [{'name':name,'bbox':[xmin,ymin,xmax,ymax]},...]
    :param ann_path:
    :return:
    '''
    from xml.dom import minidom
    tree = minidom.parse(ann_path)
    root = tree.documentElement
    objects = root.getElementsByTagName('object')
    bboxes = []
    size = root.getElementsByTagName('size')
    H = size.getElementsByTagName('height')[0].data
    W = size.getElementsByTagName('width')[0].data
    H = float(H)
    W = float(W)
    for object in objects:
        name = object.getElementsByTagName('name')[0].childNodes[0].data
        bndboxes = object.getElementsByTagName('bndbox')
        for bndbox in bndboxes:
            xmin = int(bndbox.getElementsByTagName('xmin')[0].childNodes[0].data)
            ymin = int(bndbox.getElementsByTagName('ymin')[0].childNodes[0].data)
            xmax = int(bndbox.getElementsByTagName('xmax')[0].childNodes[0].data)
            ymax = int(bndbox.getElementsByTagName('ymax')[0].childNodes[0].data)
            bboxes.append({'name': name, 'bbox': [xmin, ymin, xmax, ymax]})
    return bboxes, H, W

def xml_parser_obb(ann_path):
    '''
    return bboxes [{'name':name,'bbox':[tlx, tly, trx, try, brx, bry, blx, bly, obbw, obbh]},...]
    :param ann_path:
    :return:
    '''
    from xml.dom import minidom
    tree = minidom.parse(ann_path)
    root = tree.documentElement
    objects = root.getElementsByTagName('object')
    bboxes = []
    witdh = int(root.getElementsByTagName('width')[0].childNodes[0].data)
    height = int(root.getElementsByTagName('height')[0].childNodes[0].data)
    for object in objects:
        name = object.getElementsByTagName('name')[0].childNodes[0].data
        points = object.getElementsByTagName('points')
        for point in points:
            tlx = float(point.getElementsByTagName('point')[1].childNodes[0].data.split(',')[0])
            tly = float(point.getElementsByTagName('point')[1].childNodes[0].data.split(',')[1])
            trx = float(point.getElementsByTagName('point')[2].childNodes[0].data.split(',')[0])
            try_ = float(point.getElementsByTagName('point')[2].childNodes[0].data.split(',')[1])
            brx = float(point.getElementsByTagName('point')[3].childNodes[0].data.split(',')[0])
            bry = float(point.getElementsByTagName('point')[3].childNodes[0].data.split(',')[1])
            blx = float(point.getElementsByTagName('point')[4].childNodes[0].data.split(',')[0])
            bly = float(point.getElementsByTagName('point')[4].childNodes[0].data.split(',')[1])
            obbw = math.sqrt((tlx - trx) ** 2 + (tly - try_) ** 2)
            obbh = math.sqrt((tlx - blx) ** 2 + (tly - bly) ** 2)
            tlx /= witdh
            tly /= height
            trx /= witdh
            try_ /= height
            brx /= witdh
            bry /= height
            blx /= witdh
            bly /= height
            bboxes.append({'name': name, 'bbox': [tlx, tly, trx, try_, brx, bry, blx, bly, obbw, obbh]})
    return bboxes

def xml2yolo(xml_root, yolo_root, xml_parser=xml_parser,name_catid=None):
    """
    convert the xml annotation to yolo format
    :param xml_root:
    :param yolo_root:
    :return:
    """
    if name_catid is None:
        name_catid = {}
    for root, dirs, files in os.walk(xml_root):
        for file in files:
            if file.endswith('.xml'):
                xml_path = os.path.join(root, file)
                obbs = xml_parser(xml_path)
                with open(os.path.join(yolo_root, 'labels', file.split('.')[0] + '.txt'), 'w') as f:
                    for obb in obbs:
                        name = obb['name']
                        if name not in name_catid:
                            name_catid[name] = len(name_catid)
                        cat = name_catid[name]
                        bbox = obb['bbox']
                        if len(bbox) > 8:
                            bbox = bbox[:8]
                        bbox = ' '.join([str(b) for b in bbox])
                        f.write(str(cat) + ' ' + bbox + '\n')



def tif2jpg(tif_root, jpg_root):
    from PIL import Image
    for root, dirs, files in os.walk(tif_root):
        for file in files:
            if file.endswith('.tif'):
                tif_path = os.path.join(root, file)
                img = Image.open(tif_path)
                img = img.convert('RGB')
                img.save(os.path.join(jpg_root, file.split('.')[0] + '.jpg'))




import torch
def tensor_mem(tensor, factor=1e6):
    tensor = torch.randn(3, 4, 5)
    # 计算张量中元素的数量
    num_elements = tensor.numel()
    # 获取单个元素的字节大小
    element_size = tensor.element_size()
    # 计算总显存大小
    total_memory = num_elements * element_size  / 1e6
    return total_memory

import torch
import gc

def find_all_tensors():
    # 获取当前PyTorch会话中的所有对象
    for obj in gc.get_objects():
        if torch.is_tensor(obj):
            yield obj

def get_tensor_memory(tensor):
    # 计算张量占用的显存大小
    return tensor.element_size() * tensor.nelement()

def find_max_memory_tensor():
    max_memory_tensor = None
    max_memory_size = 0

    # 获取所有张量
    all_tensors = find_all_tensors()

    # 找到占用显存最大的张量
    for tensor in all_tensors:
        memory_size = get_tensor_memory(tensor)
        if memory_size > max_memory_size:
            max_memory_size = memory_size
            max_memory_tensor = tensor

    return max_memory_tensor